@@ -73,7 +73,7 @@ all other classes are runtime bindings::
     inject.configure(my_config)
 
 """
-__version__ = '4.2.0'
+__version__ = '4.3.0'
 __author__ = 'Ivan Korobkov <ivan.korobkov@gmail.com>'
 __license__ = 'Apache License 2.0'
 __url__ = 'https://github.com/ivan-korobkov/python-inject'
@@ -85,8 +85,9 @@ import sys
 import threading
 import types
 from functools import wraps
-from typing import (Any, Callable, Dict, Generic, Hashable, Optional, Type,
-                    TypeVar, Union, get_type_hints, overload)
+from typing import (Any, Awaitable, Callable, Dict, Generic, Hashable,
+                    Optional, Type, TypeVar, Union, cast, get_type_hints,
+                    overload)
 
 _NEW_TYPING = sys.version_info[:3] >= (3, 7, 0)  # PEP 560
 _RETURN = 'return'
@@ -111,6 +112,11 @@ Provider = Constructor
 BinderCallable = Callable[['Binder'], None]
 
 
+class ConstructorTypeError(TypeError):
+    def __init__(self, constructor: Callable, previous_error: TypeError):
+        super(ConstructorTypeError, self).__init__("%s raised an error: %s" % (constructor, previous_error))
+
+
 class Binder(object):
     _bindings: Dict[Binding, Constructor]
 
@@ -224,7 +230,11 @@ class Injector(object):
                 raise InjectorException(
                     'Cannot create a runtime binding, the key is not callable, key=%s' % cls)
 
-            instance = cls()
+            try:
+                instance = cls()
+            except TypeError as previous_error:
+                raise ConstructorTypeError(cls, previous_error)
+
             self._bindings[cls] = lambda: instance
 
             logger.debug(
@@ -271,12 +281,22 @@ class _ParameterInjection(Generic[T]):
         self._name = name
         self._cls = cls
 
-    def __call__(self, func: Callable[..., T]) -> Callable[..., T]:
+    def __call__(self, func: Callable[..., Union[T, Awaitable[T]]]) -> Callable[..., Union[T, Awaitable[T]]]:
+        if inspect.iscoroutinefunction(func):
+            @wraps(func)
+            async def async_injection_wrapper(*args: Any, **kwargs: Any) -> T:
+                if self._name not in kwargs:
+                    kwargs[self._name] = instance(self._cls or self._name)
+                async_func = cast(Callable[..., Awaitable[T]], func)
+                return await async_func(*args, **kwargs)
+            return async_injection_wrapper
+        
         @wraps(func)
         def injection_wrapper(*args: Any, **kwargs: Any) -> T:
             if self._name not in kwargs:
                 kwargs[self._name] = instance(self._cls or self._name)
-            return func(*args, **kwargs)
+            sync_func = cast(Callable[..., T], func)
+            return sync_func(*args, **kwargs)
 
         return injection_wrapper
 
@@ -287,13 +307,29 @@ class _ParametersInjection(Generic[T]):
     def __init__(self, **kwargs: Any) -> None:
         self._params = kwargs
 
-    def __call__(self, func: Callable[..., T]) -> Callable[..., T]:
+    def __call__(self, func: Callable[..., Union[Awaitable[T], T]]) -> Callable[..., Union[Awaitable[T], T]]:
         if sys.version_info.major == 2:
             arg_names = inspect.getargspec(func).args
         else:
             arg_names = inspect.getfullargspec(func).args
         params_to_provide = self._params
 
+        if inspect.iscoroutinefunction(func):
+            @wraps(func)
+            async def async_injection_wrapper(*args: Any, **kwargs: Any) -> T:
+                provided_params = frozenset(
+                    arg_names[:len(args)]) | frozenset(kwargs.keys())
+                for param, cls in params_to_provide.items():
+                    if param not in provided_params:
+                        kwargs[param] = instance(cls)
+                async_func = cast(Callable[..., Awaitable[T]], func)
+                try:
+                    return await async_func(*args, **kwargs)
+                except TypeError as previous_error:
+                    raise ConstructorTypeError(func, previous_error)
+
+            return async_injection_wrapper
+
         @wraps(func)
         def injection_wrapper(*args: Any, **kwargs: Any) -> T:
             provided_params = frozenset(
@@ -301,7 +337,11 @@ class _ParametersInjection(Generic[T]):
             for param, cls in params_to_provide.items():
                 if param not in provided_params:
                     kwargs[param] = instance(cls)
-            return func(*args, **kwargs)
+            sync_func = cast(Callable[..., T], func)
+            try:
+                return sync_func(*args, **kwargs)
+            except TypeError as previous_error:
+                raise ConstructorTypeError(func, previous_error)
         return injection_wrapper
 
 
@@ -407,7 +447,10 @@ def autoparams(*selected: str) -> Callable:
             pass
     """
     def autoparams_decorator(fn: Callable[..., T]) -> Callable[..., T]:
-        types = get_type_hints(fn)
+        if inspect.isclass(fn):
+            types = get_type_hints(fn.__init__)
+        else:
+            types = get_type_hints(fn)
 
         # Skip the return annotation.
         types = {name: typ for name, typ in types.items() if name != _RETURN}
